c------------------------------------------------------------
c Programm-Autor: Juergen Prenzel
c                 Institut fuer Bodenkunde und Waldernaehrung
c                 Buesgenweg 2
c                 3400 Goettingen
c------------------------------------------------------------
c sr HEX
c berechnet austauschbaren Wasserstoff
c
c Argumentliste:
c 1.) nco  (input, integer, variable)         =   Anzahl der Al-OH-Komplexe
c 2.) ix   (input, integer, array, dim=nco)
c          ix(i) = Zahl der Al-Atome im Komplex i
c 3.) iy   (input, integer, array, dim=nco)
c          iy(i) = Zahl der OH-Gruppen im Komplex i
c 4.) pK   (input, real, array, dim=nco)
c          pK(i) = -log10(K), mit K=Gleichgewichtskonstante der Komplex-
c          zerfallsreaktion
c 5.) pH1  (input, real, variable)     = pH des Extraktionsmittels
c 6.) pH2  (input, real, variable)     = pH des Extraktes
c 7.) gwt  (input, real, variable)     = Gewicht der Probe in g
c 8.) vol  (input, real, variable)     = Volumen des Extraktes
c 9.) exal (input, real, variable)     = austauschbares Al [mg(Al)/kg(tB)!
c10.) cal  (output, real, variable)    = Al3+ im Extrakt [mol/l]
c11.) exh  (output, real, variable)    = austauschbare H-Ionen [mmol(H)/kg(tB)!
c12.) ier  (output, integer, variable) = Fehlerindikator
c          ier=0 wenn keine Fehler, ier=1, wenn keine Konvergenz
      subroutine hex(nco,ix,iy,pk,ph1,ph2,gwt,vol,exal,cal,exh,ier)
      implicit double precision (a-h,o-z)
      dimension ix(1),iy(1),pk(1)
c
c Initialisierungen
c
      x=0.
      soh=0.
      if(exal.le.0.) go to 500
      pg3=dlog(actv(3))
      flogln=dlog(10.d0)
      poh=14.d0-ph2
      poh=-poh*flogln
      alt=exal*gwt/vol/27.e6
c
c Suche eines vernuenftigen Startpunktes fuer Newton-Verfahren
c
      x=alt
200   continue
      f=x-alt
      px=dlog(x)
      df=1.d0
      do 201 l=1,nco
      pgc=dlog(actv(3*ix(l)-iy(l)))
      pcoc=ix(l)*(pg3+px) + iy(l)*poh - (-pk(l)*flogln) - pgc
      if(pcoc.gt.dlog(alt)) then
           x=x/2.
           go to 200
      endif
201   continue
c
c Newton-Iteration
c
300   continue
      do 301 l1=1,100
      f=x-alt
      px=dlog(x)
      df=1.d0
      do 302 l2=1,nco
      pgc=dlog(actv(3*ix(l2)-iy(l2)))
      pcoc=ix(l2)*(pg3+px) + iy(l2)*poh - (-pk(l2)*flogln) - pgc
      coc=hxclim(pcoc)
      f=f+ix(l2)*coc
      df=df+(ix(l2)*ix(l2))*coc/x
302   continue
      if (dabs(f/alt).lt.1.d-12) go to 400
301   x=x-f/df
      ier=1
      return
c
c Berechnung der austauschbaren H-Ionen
c
400   soh=0.
      px=dlog(x)
      do 401 l=1,nco
      pgc=dlog(actv(3*ix(l)-iy(l)))
      pcoc=ix(l)*(pg3+px) + iy(l)*poh - (-pk(l)*flogln) - pgc
      coc=hxclim(pcoc)
401   soh=soh+iy(l)*coc
500   exh=(10.**(-ph2) - 10.**(-ph1))/actv(1)  -  soh
      exh=exh*vol/gwt*1.e6
      cal=x
      ier=0
      return
      end
c------------------------------------------------------------
c Programm-Autor: Juergen Prenzel
c                 Institut fuer Bodenkunde und Waldernaehrung
c                 Buesgenweg 2
c                 3400 Goettingen
c------------------------------------------------------------
c function ACTV
c Aktivitaetskoeffizient nach der Davies-Gleichung
c in einer 1-molaren NH4Cl - Loesung
c
      double precision function actv(iz)
      actv=10.d0**(-0.1d0*(iz*iz))
      return
      end
c------------------------------------------------------------
c Programm-Autor: Juergen Prenzel
c                 Institut fuer Bodenkunde und Waldernaehrung
c                 Buesgenweg 2
c                 3400 Goettingen
c------------------------------------------------------------
c function hxclim                                                       COC00010
c   berechnet Komplex-Konzentration aus deren Logarithmus unter         COC00020
c   Beachtung von Limits                                                COC00030
c                                                                       COC00040
      double precision function hxclim(pcoc)                            COC00050
      double precision pcoc                                             COC00060
      if (pcoc.gt.115.12925d0) then                                     COC00070
         hxclim =1.d50                                                  COC00080
      else if (pcoc.lt.-115.12925d0) then                               COC00090
         hxclim =0.d0                                                   COC00100
      else                                                              COC00110
         hxclim =DEXP(PCOC)                                             COC00120
      endif                                                             COC00130
      return                                                            COC00140
      end
c------------------------------------------------------------
c Programm-Autor: Juergen Prenzel
c                 Institut fuer Bodenkunde und Waldernaehrung
c                 Buesgenweg 2
c                 3400 Goettingen
c------------------------------------------------------------
c hp HEX$
c testet die sr HEX (austauschbarer Wasserstoff)
c
      program hex$
      implicit double precision (a-h,o-z)
      dimension ix(50),iy(50),pk(50)
      integer ncase/0/
      character titel*80,vor*1
      open(unit=8,file='hex.liste',status='new')
      open(unit=9,file='hex.input',blank='zero',status='old')
      open(unit=10,file='hex.parm',status='old')
      vol=0.1
      write(8,120)
120   format(/////' Programm HEX$ (Berechnung von austauschbaren'
     b,' H-Ionen)'///)
c
c Parameter lesen
c
      read(10,*) nco
      write(8,*) '          Tabelle der verwendeten Al-OH-Komplexe und'
     &//' ihrer Gleichgewichtskonstanten'
      write(8,*)
      do 201 l=1,nco
      read(10,*) ix(l),iy(l),pk(l)
      iz=3*ix(l) - iy(l)
201   write(8,105) l,ix(l),iy(l),iz,ix(l),iy(l),pk(l)
105   format (1x,i2,'.)     (Al',i2,'(OH)',i2,')',SP,i2,SS,'     =     '
     &,i2,'  Al3+  +  ',i2,'  OH-     ;     -log K  =  ',f10.4)
      vor=' '
200   ncase=ncase+1
c
c Daten lesen
c
      read(9,101,end=800) titel
      write(8,103)vor,ncase, titel
103   format(a1,/////,' Fall Nr. ',i3,', Titelkarte: ',a80)
      vor='1'
      read(9,102,end=800) ph1,ph2,exal,gwt
102   format(/2f3.2,t50,f7.3,7x,f4.2)
      alt=exal*gwt/vol/27.e6
      write(8,106) pH1,pH2,gwt,vol,exal,alt
106   format(//' Eingangsdaten fuer Unterprogramm HEX: '/
     &' pH der Loesung vor der Extraktion:',t50,f7.4/
     &' pH der Loesung nach der Extraktion:',t50,f7.4/
     &' Probengewicht:',t50,f7.4,t60,'[g]'/
     &' Menge der Loesung:',t50,f7.5,t60,'[l]'/
     &' austauschbares Aluminium:',t50,f10.4,t62,'[mg(Al)/kg(tB)]'
     &,'   =   ',g10.4,'   [mol/l]')
c
c austauschbaren Wasserstoff berechnen (sr HEX)
c
      call hex(nco,ix,iy,pk,ph1,ph2,gwt,vol,exal,cal,exh,ier)
      write(8,107) ier,exh,cal
107   format('0Ergebnisse von Unterprogramm HEX:'/
     &' IER (Fehlerindikator) = ',t50,i3/
     &,' austauschbare H-Ionen: ',t50,g10.4,t65,'[mg(H)/kg(tB)]'/
     &' Al3+ - Konzentration:',t50,g10.4,t65,'[mol/l]')
      if (exal.le.0.) go to 200
c
c Aufschluesselung der Al-Formen
c
      write(8,108)
108   format('0Aufschluesselung der Al - Formen:')
      proz=cal/alt*100.
      write(8,104) 0,1,0,3,cal,proz,actv(3)
104   format (1x,i2,'.)    [(Al',i2,'(OH)',i2,')',SP,i2,SS,']    =    '
     &,t50,g10.4,t65,'[mol/l]      =     ',g10.4,' % von Al-t,'
     &,'   g   =   ',g10.4)
      flogln=dlog(10.d0)
      pg3=dlog(actv(3))
      poh=14.-ph2
      poh=-poh*flogln
      altx=cal
      roh=0.
      soh=0.
      pcal=dlog(cal)
      do 401 l=1,nco
      pgc=dlog(actv(3*ix(l)-iy(l)))
      pcoc=ix(l)*(pg3+pcal) + iy(l)*poh - (-pk(l)*flogln) - pgc
      coc=hxclim(pcoc)
      proz=coc/alt*100.
      soh=soh+coc*iy(l)
      altx=altx+ix(l)*coc
      iz=3*ix(l)-iy(l)
      write(8,104) l,ix(l),iy(l),iz,coc,proz,actv(iz)
401   continue
      roh=soh/alt
      psp= -dlog10(cal*actv(3)) + 3*(14.-pH2)
      write(8,109) roh,psp
109   format('0Daraus ergibt sich:'/
     &' rOH [mol(OH)/mol(Al)]  =  ',t50,f7.5/
     &' pAl(OH)3   =   ',t50,f8.4)
c
c Kontrollrechnung
c
      hLx=soh + exh*gwt/vol*1.e-6 + 10.d0**(-ph1)/actv(1)
      if (hLx.gt.0.d0) then
         phLx=-dlog10(hLx*actv(1))
      else
         phLx=99.d0
      endif
      dph =(phLx-ph2)/ph2*100.
      dalt=(altx-alt)/alt*100.
      write(8,110) altx,dalt,phlx,dph
110   format('0Kontrollrechnung (Zurueckberechnung von Eingangsdaten):'/
     &' Al-t   =   ',g10.4,t30,'(Abweichung ',g10.4,'%)'/
     &' pH-2   =   ',f8.5,t30,'(Abweichung ',g10.4,'%)')
      go to 200
800   stop
101   format(a80)
      end















